<?php


namespace IPS\dp42sulk\modules\admin\manage;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * index
 */
class _index extends \IPS\Dispatcher\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;
    	
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'index_manage', 'dp42sulk' );
		parent::execute();
	}
	
	/**
	 * Manage
	 *
	 * @return	void
	 */
	protected function manage()
	{		
		/* Build where */
        $where = array( "nexus_packages_products.p_lkey = ?", 'sulk' );          
        
        /* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'nexus_packages_products', \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index' ), array( $where ) );

        $table->langPrefix = 'dp42sulk_products__';
        $table->limit = 20;
        
        /* Joins */                
		$table->joins = array(
			array(
				'select' => 'np.*, np.p_id as np_id',
				'from'   => array( 'nexus_packages', 'np' ),
				'where'  => "nexus_packages_products.p_id = np.p_id",
                ),              
		);  
        
		/* Column stuff */
		$table->include    = array( 'p_name', 'p_stock', 'used_keys' );
        $table->noSort	   = array( 'p_name' );
		$table->mainColumn = 'p_name';
        
		$table->widths  = array(
            'p_name'    => 55   
		);

        /* Default sort options */
		$table->sortBy = $table->sortBy ?: 'p_name';
		$table->sortDirection = $table->sortDirection ?: 'asc';
		
        /* Search */
		$table->quickSearch = 'p_name';

		/* Custom parsers */
		$table->parsers = array(
			'p_name' => function( $val, $row )
			{
                if( \IPS\Member::loggedIn()->language()->checkKeyExists( 'nexus_package_' . $row['p_id'] ) )
                {
                    $packageName = \IPS\Member::loggedIn()->language()->get( 'nexus_package_' . $row['p_id'] );
                }
                else
                {
                    return \IPS\Member::loggedIn()->language()->get( 'dp42sulk_removed_product' );
                }
                
                return "<a href='" . \IPS\Http\Url::internal( "app=nexus&module=store&controller=packages&subnode=1&do=form&id={$row['p_id']}", 'admin' ) . "'>" . $packageName . "</a>";
			},
			'p_stock' => function( $val, $row )
			{
                return $val;
			}, 
			'used_keys' => function( $val, $row )
			{
                if( $row['p_id'] )
                {
                    return \IPS\Db::i()->select( 'COUNT(*)', 'dp42sulk_keys', array( 'l_product_id = ? AND l_used_date <> ?', $row['p_id'], 0 ) )->first();
                }
                return '-';
			} 
		);
        
        $table->rowButtons = function( $row )
		{
            $return = array();
            
            if( $row['p_id'] )
            {
                $return['view_keys'] = array(
                	'icon'		=> 'search',
                	'title'		=> 'dp42sulk_products__view_keys',
                	'link'		=> \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=view&id=' . $row['p_id'] ),
                );
                
                $return['add_keys'] = array(
                	'icon'		=> 'download',
                	'title'		=> 'dp42sulk_products__add_keys',
                	'link'		=> \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=import&id=' . $row['p_id'] ),
                );
        
                $return['export_unused']= array(
        			'icon' => 'upload',
        			'link' => \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=exportunused&id=' . $row['p_id'] )->csrf(),
        			'title' => 'dp42sulk_products__exportunused',
        		);
                    
                $return['refresh_stock'] = array(
                	'icon'		=> 'refresh',
                	'title'		=> 'dp42sulk_products__refresh_stock',
                	'link'		=> \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=refresh&id=' . $row['p_id'] )->csrf(),
                );
            }
            
			return $return;
		};
        
        /* Action Buttons */
		\IPS\Output::i()->sidebar['actions'][] = array(
			'icon'	=> 'refresh',
			'title'	=> 'dp42sulk_products__refresh_stock',
			'link'	=> \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=refresh' )->csrf(),
            'data'  => array( 'confirm' => '' )
		);

		/* Display */
        \IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack( 'menu__dp42sulk_manage_index' );
		\IPS\Output::i()->output = \IPS\Member::loggedIn()->language()->addToStack( 'dp42sulk_products__header_info' ) . (string) $table . \IPS\Application::load( 'dp42sulk' )->cpr();
	}
    
    public function refresh()
    {
        \IPS\Session::i()->csrfCheck();	
        
        $id = \IPS\Request::i()->id ? \IPS\Request::i()->id : 0;
        
        \IPS\dp42sulk\Settings::updateStock( $id );
        
        \IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index' ), 'dp42sulk_products__refreshed_done' );
    }
    
    public function import()
    {
        /* Got the package details? */
		try
		{
			$package = \IPS\nexus\Package::load( \IPS\Request::i()->id );
		}
		catch( \OutOfRangeException $e )
		{
            \IPS\Output::i()->error( 'node_error', 'DP44SULK003', 404, '' );
		}
        
        $form = new \IPS\Helpers\Form( 'import_keys', 'dp42sulk_button_import' );
        
        $form->add( new \IPS\Helpers\Form\TextArea( 'dp42sulk_codes', NULL, TRUE, array( 'rows' => 25, 'class' => 'ipsField_codeInput' ), NULL, NULL, NULL, 'dp42sulk_codes' ) );

        if( $values = $form->values() )
		{
            $keys = explode( "\n", trim( $values['dp42sulk_codes'] ) ); 
            $cnt  = 0;
            
    		if( \count( $keys ) )
            {
                foreach( $keys as $key )
                {
                    $key = trim( str_replace( ' ', '', $key ) );
                    
                    if( mb_strlen( $key ) )
                    {
                        try
                		{
                			$record	= \IPS\Db::i()->select( '*', 'dp42sulk_keys', array( 'l_key = ? AND l_product_id = ?', (string) $key, $package->id ) )->first();
                		}
                		catch( \UnderflowException $e )
                		{
                            $sulk            = new \IPS\dp42sulk\Key;
                            $sulk->key        = $key;
                            $sulk->product_id = $package->id;
                            $sulk->save();
                            
                            $cnt++;	
                		}
                    }
                }
                
                /* Update stock */
                if( $cnt )
                {
                    \IPS\dp42sulk\Settings::updateStock( $package->id );
                }
            }  
            
            \IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index' ), \IPS\Member::loggedIn()->language()->addToStack( 'dp42sulk_products__imported_done', FALSE, array( 'sprintf' => array( $cnt, $package->_title ) ) ) );          
        }
        
		/* Display */
        \IPS\Output::i()->title         = \IPS\Member::loggedIn()->language()->addToStack( 'menu__dp42sulk_manage_import' );
		\IPS\Output::i()->breadcrumb[]	= array( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__dp42sulk_manage_index' ) );
        \IPS\Output::i()->breadcrumb[]	= array( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=import&id=' . $package->id )->csrf(), \IPS\Member::loggedIn()->language()->addToStack( 'dp42sulk_products__importing_keys_into', FALSE, array( 'sprintf' => array( $package->_title ) ) ) );

		\IPS\Output::i()->output = (string) $form . \IPS\Application::load( 'dp42sulk' )->cpr();
    }
    
    /**
     * Multimod
     *
     * Handles multiple editing of records at once.
     *
     * @return    void
     */
    protected function multimod()
    {
        \IPS\Session::i()->csrfCheck();

        if( !isset( \IPS\Request::i()->multimod ) || ! \is_array( \IPS\Request::i()->multimod ) || ! \count( \IPS\Request::i()->multimod ) )
        {
            \IPS\Output::i()->error( 'dp43sulk_nothing_selected', 'DP44SULK007', 403 );
        }

        \IPS\Db::i()->delete( \IPS\dp42sulk\Key::$databaseTable, array( array( \IPS\Db::i()->in( 'l_id', array_keys( \IPS\Request::i()->multimod ) ) ), array( 'l_used_date=?', 0 ) ) );
        \IPS\Output::i()->redirect( \IPS\Http\Url::internal( "app=dp42sulk&module=manage&controller=index&do=view&id=" . \IPS\Request::i()->id )->csrf(), 'deleted' );
    }
    
    public function view()
    {
        /* Got the package details? */
		try
		{
			$package = \IPS\nexus\Package::load( \IPS\Request::i()->id );
		}
		catch( \OutOfRangeException $e )
		{
            \IPS\Output::i()->error( 'node_error', 'DP44SULK004', 404, '' );
		}
        
		/* Build where */
        $where = array( "l_product_id = ?", $package->id );          
        
        /* Create the table */
		$table = new \IPS\Helpers\Table\Db( 'dp42sulk_keys', \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=view&id=' . $package->id ), array( $where ) );
        $table->tableTemplate = array( \IPS\Theme::i()->getTemplate( 'keys' ), 'keyTable' );
        $table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'keys' ), 'keyRow' );
        $table->rowClasses = array( 'key_content' => array( 'ipsTable_wrap' ) );
        
        $table->langPrefix = 'dp42sulk_keys__';
        $table->limit = 20;
        
		/* Column stuff */
		$table->include    = array( 'l_id', 'l_key', 'purchase_info', 'l_used_date', '_l_used_date' );
        $table->noSort	   = array( 'l_key' );
		$table->mainColumn = 'l_key';
        
		$table->widths  = array(
            'l_key'         => 50,
            'purchase_info' => 30,
            'l_used_date'   => 20,    
		);

        /* Default sort options */
		$table->sortBy = $table->sortBy ?: 'l_used_date';
		$table->sortDirection = $table->sortDirection ?: 'desc';
		
        /* Search */
		$table->quickSearch = 'l_key';

		/* Custom parsers */
		$table->parsers = array(
			'_l_used_date' => function( $val, $row )
			{
                return $row['l_used_date'];
			},
			'l_key' => function( $val, $row )
			{
                return $val;
			},
            'purchase_info' => function($val, $row ) 
            {
        		/* Renew */
                if( $row['l_purchase_id'] && $row['l_invoice_id'] )
                {
                    try
            		{			
            			$lic      = \IPS\Db::i()->select( '*', 'nexus_licensekeys', array( 'lkey_type = ? AND lkey_purchase = ?', 'sulk', $row['l_purchase_id'] ) )->first();
                        
                        $purchase = \IPS\nexus\Purchase::load( $lic['lkey_purchase'] );
    
                        $by       = '*';
                        if( \is_int( $purchase->member->member_id ) )
                        {
                            $bought = \IPS\Member::load( $purchase->member->member_id );
                            $by     = "<a href='" . $bought->acpUrl() . "'>" . htmlentities( $bought->name, ENT_DISALLOWED, 'UTF-8', FALSE ) . "</a>";
                        }

                        return \IPS\Member::loggedIn()->language()->addToStack( 'dp42sulk_keys__purchase_info_details_renew', false, array( 'htmlsprintf' => array( $purchase->name, $by ) ) ); 
            		}
            		catch( \UnderflowException $e )
            		{
                    } 
                }
                else
                {
                    /* Main purchase maybe? */
                    try
            		{			
            			$lic      = \IPS\Db::i()->select( '*', 'nexus_licensekeys', array( 'lkey_type = ? AND lkey_key = ?', 'sulk', $row['l_key'] ) )->first();
                        
                        $purchase = \IPS\nexus\Purchase::load( $lic['lkey_purchase'] );
    
                        $by       = '*';
                        if( \is_int( $purchase->member->member_id ) )
                        {
                            $bought = \IPS\Member::load( $purchase->member->member_id );
                            $by     = "<a href='" . $bought->acpUrl() . "'>" . htmlentities( $bought->name, ENT_DISALLOWED, 'UTF-8', FALSE ) . "</a>";
                        }
                        
                        return \IPS\Member::loggedIn()->language()->addToStack( 'dp42sulk_keys__purchase_info_details', false, array( 'htmlsprintf' => array( $purchase->name, $by ) ) ); 
            		}
            		catch( \UnderflowException $e )
            		{
                    }
                    
                    return '- - -'; 
                }           
            },
			'l_used_date' => function( $val, $row )
			{
                return $val ? \IPS\DateTime::ts( $val ) : '- - -';
			}, 
		);
        
        $table->rowButtons = function( $row )
		{
            $return = array();
            
            if( $row['l_used_date'] )
            {
                $return['restore'] = array(
            	'icon'		=> 'level-up',
            	'title'		=> 'dp45restore_key_restore_title',
            	'link'		=> \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=restore&id=' . $row['l_id'] )->csrf(),
            	'data'      => array(
            		'delete' => '',
                    'delete-message' => \IPS\Member::loggedIn()->language()->addToStack( 'dp45restore_key_restore_title_sub' ),
            		'delete-warning' => \IPS\Member::loggedIn()->language()->addToStack( 'dp45restore_key_restore_title_confirm' )
            	               )
                );
            }
            
			return $return;
		};
        
		\IPS\Output::i()->sidebar['actions']['add_keys'] = array(
			'icon' => 'download',
			'link' => \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=import&id=' . $package->id )->csrf(),
			'title' => 'dp42sulk_products__add_keys',
		);
		
        \IPS\Output::i()->sidebar['actions']['refresh_stock'] = array(
			'icon' => 'refresh',
			'link' => \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=refresh&id=' . $package->id )->csrf(),
			'title' => 'dp42sulk_products__refresh_stock',
		);
        
        \IPS\Output::i()->sidebar['actions']['export_unused'] = array(
			'icon' => 'upload',
			'link' => \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=exportunused&id=' . $package->id )->csrf(),
			'title' => 'dp42sulk_products__exportunused',
		);
        
		/* Display */
        \IPS\Output::i()->title  = \IPS\Member::loggedIn()->language()->addToStack( 'menu__dp42sulk_viewing_keys_for_package', FALSE, array( 'sprintf' => array( $package->_title ) ) );
		\IPS\Output::i()->breadcrumb[]	= array( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__dp42sulk_manage_index' ) );
        \IPS\Output::i()->breadcrumb[]	= array( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=view&id=' . $package->id ), \IPS\Output::i()->title );
		\IPS\Output::i()->output = (string) $table . \IPS\Application::load( 'dp42sulk' )->cpr();
    }
    
   public function delete()
    {
		\IPS\Session::i()->csrfCheck();	
        
        /* Make sure the user confirmed the deletion */
		\IPS\Request::i()->confirmedDelete();
        
        /* Load item */
		try
		{
            $item = \IPS\dp42sulk\Key::load( \IPS\Request::i()->id );
		}
		catch( \Exception $e )
		{
			\IPS\Output::i()->error( 'dp42sulk_key_not_found', 'DP44SULK001', 404, '' );
		}

        /* Key is used? */        
        if( $item->used_date )
		{
            \IPS\Output::i()->error( 'dp42sulk_key_used_cant_delete', 'DP44SULK002', 403, '' );			
		}
        
		/* Delete */
		\IPS\Session::i()->log( 'acplog__dp42sulk_key_item_deleted' );
		$item->delete();
        
        /* Update stock */
        \IPS\dp42sulk\Settings::updateStock( $item->product_id );
        
		/* Boink */
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=view&id=' . $item->product_id ), 'deleted' );
    }
    
    public function exportunused()
    {
        \IPS\Session::i()->csrfCheck();	
        
        /* Got the package details? */
		try
		{
			$package = \IPS\nexus\Package::load( \IPS\Request::i()->id );
		}
		catch( \OutOfRangeException $e )
		{
            \IPS\Output::i()->error( 'node_error', 'DP44SULK005', 404, '' );
		}
        
        /* Get and check if we have any unused keys */
    	$keys = array();
        foreach( new \IPS\Patterns\ActiveRecordIterator( \IPS\Db::i()->select( '*', 'dp42sulk_keys', array( 'l_product_id = ' . $package->id . ' AND l_used_date = 0' ), 'l_id DESC' ), 'IPS\dp42sulk\Key' ) as $key )
    	{
    		$keys[ $key->id ] = $key->key;
    	}

        if( empty( $keys ) )
        {
            \IPS\Output::i()->error( 'dp42sulk_key_error_no_unused_keys', 'DP44SULK006', 404, '' );    
        }
        
        if( \IPS\Member::loggedIn()->language()->checkKeyExists( 'nexus_package_' . $package->id ) )
        {
            $packageName = \IPS\Member::loggedIn()->language()->get( 'nexus_package_' . $package->id );
        }
        else
        {
            $packageName = \IPS\Member::loggedIn()->language()->get( 'dp42sulk_removed_product' );
        }        
        
        /* Still here? Export it! */
        \IPS\Output::i()->sendOutput( implode( "\n", $keys ), 200, 'text/plain', array( 'Content-Disposition' => 'attachment; filename="unusedkeys - ' . $packageName . '.txt"' ) );
        
		exit;
		
       
        
		/* Display */
        \IPS\Output::i()->title         = \IPS\Member::loggedIn()->language()->addToStack( 'menu__dp42sulk_export_unused' );
		\IPS\Output::i()->breadcrumb[]	= array( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index' ), \IPS\Member::loggedIn()->language()->addToStack( 'menu__dp42sulk_manage_index' ) );
        \IPS\Output::i()->breadcrumb[]	= array( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=exportunused&id=' . $package->id )->csrf(), \IPS\Member::loggedIn()->language()->addToStack( 'dp42sulk_products__exportunused', FALSE, array( 'sprintf' => array( $package->_title ) ) ) );

		\IPS\Output::i()->output = (string) $form . \IPS\Application::load( 'dp42sulk' )->cpr();
    }
    
   public function restore()
    {
		\IPS\Session::i()->csrfCheck();	
        
        /* Make sure the user confirmed the deletion */
		\IPS\Request::i()->confirmedDelete();
        
        /* Load item */
		try
		{
            $item = \IPS\dp42sulk\Key::load( \IPS\Request::i()->id );
		}
		catch( \Exception $e )
		{
			\IPS\Output::i()->error( 'dp42sulk_key_not_found', 'DP44SULK009', 404, '' );
		}

        /* Key isn't used? */        
        if( ! $item->used_date )
		{
            \IPS\Output::i()->error( 'dp42sulk_key_used_cantrestore', 'DP44SULK008', 403, '' );			
		}
        
		/* Delete */
		\IPS\Session::i()->log( 'acplog__dp42sulk_key_item_restored' );
		$item->used_date = 0;
        $item->save();
        
        /* Update stock */
        \IPS\dp42sulk\Settings::updateStock( $item->product_id );
        
		/* Boink */
		\IPS\Output::i()->redirect( \IPS\Http\Url::internal( 'app=dp42sulk&module=manage&controller=index&do=view&id=' . $item->product_id ), 'acplog__dp42sulk_key_item_restored' );
    }
}